/**
 * Written by Jason Heddings: http://mapit.mozdev.org/
 * License: http://creativecommons.org/licenses/by-nc-sa/2.0/
 * $Id: mapitUtil.js,v 1.8 2005/07/20 22:46:51 jheddings Exp $
 */

////////////////////////////////////////////////////////////////////////////////
// Creates a new MapItAddress from the currently selected text.

function mapitReadSelectedAddress() {
	// The following broke in 1.0.3 - eb
  // var text = window._content.__proto__.getSelection.call(window._content).toString();

	// why doesn't this work:  --jah
	// var text = window.getSelection().toString();

  // thanks to eric brunson
  var fWin = document.commandDispatcher.focusedWindow
  var text = fWin.getSelection.call(fWin).toString();

	gMapItService.log("selected text:\n" + text);
	return (text ? new MapItAddress(text) : null);
}

////////////////////////////////////////////////////////////////////////////////
// Creates a blank form.

function mapitGenerateForm(method, action) {
	gMapItService.log("generate form: " + action + " [" + method + "]\n");
  gMapItService.document = mapitGenerateDocument("about:blank");
  var form = gMapItService.document.createElement("form")
  form.setAttribute("method", method);
  form.setAttribute("action", action);
  gMapItService.document.body.appendChild(form);
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Generates a page in a new tab or window

function mapitGenerateDocument(url) {
  var settings = gMapItService.settings;
	gMapItService.log("generate document: " + settings.target + "\n");

	// check where to send the map
  var genPage = window;
  if (settings.target == "tab") {
    getBrowser().selectedTab = getBrowser().addTab(url);
  } else if (settings.target == "blank") {
    genPage = window.open(url);
  } else {
    // the new page is set to the current window
  }

  return genPage.content.document;
}

////////////////////////////////////////////////////////////////////////////////
// Creates a form input element.

function mapitGenerateInput(name, value) {
  var input = gMapItService.document.createElement("input");
  input.setAttribute("type", "hidden");
  input.setAttribute("name", name);
  input.setAttribute("value", value);
  return input;
}

////////////////////////////////////////////////////////////////////////////////
// Submits the form.

function mapitSubmit(form) {
	gMapItService.log("submit form\n");

	// This must be done to make generated content render
  const request = new XMLHttpRequest();
  request.open("GET", window.content.document.URL, true);
  request.send("");
  form.submit();
}

////////////////////////////////////////////////////////////////////////////////
// Change full state name to abbreviated one.
// TODO :jah: variants of names should be considered

function mapitAbbreviateState(state) {
  if (! state) {
    return null;
  }

  switch (state.toLowerCase()) {
    case "alabama":        return "AL";
    case "alaska":         return "AK";
    case "arizona":        return "AZ";
    case "arkansas":       return "AR";
    case "california":     return "CA";
    case "colorado":       return "CO";
    case "connecticut":    return "CT";
    case "d.c.":           return "DC";
    case "delaware":       return "DE";
    case "florida":        return "FL";
    case "georgia":        return "GA";
    case "hawaii":         return "HI";
    case "idaho":          return "ID";
    case "illinois":       return "IL";
    case "indiana":        return "IN";
    case "iowa":           return "IA";
    case "kansas":         return "KS";
    case "kentucky":       return "KY";
    case "louisiana":      return "LA";
    case "maine":          return "ME";
    case "maryland":       return "MD";
    case "massachusetts":  return "MA";
    case "michigan":       return "MI";
    case "minnesota":      return "MN";
    case "mississippi":    return "MS";
    case "missouri":       return "MO";
    case "montana":        return "MT";
    case "nebraska":       return "NE";
    case "nevada":         return "NV";
    case "new hampshire":  return "NH";
    case "new jersey":     return "NJ";
    case "new mexico":     return "NM";
    case "new york":       return "NY";
    case "north carolina": return "NC";
    case "north dakota":   return "ND";
    case "ohio":           return "OH";
    case "oklahoma":       return "OK";
    case "oregon":         return "OR";
    case "pennsylvania":   return "PA";
    case "rhode island":   return "RI";
    case "south carolina": return "SC";
    case "south dakota":   return "SD";
    case "tennessee":      return "TN";
    case "texas":          return "TX";
    case "utah":           return "UT";
    case "vermont":        return "VT";
    case "virginia":       return "VA";
    case "washington":     return "WA";
    case "west virginia":  return "WV";
    case "wisconsin":      return "WI";
    case "wyoming":        return "WY";
  }

  return state;
}

////////////////////////////////////////////////////////////////////////////////
// Generate a string for debugging.

function getDebugText() {
  var debugString = "**** BEGIN MapIt! DEBUG LOG ****\n";
	debugString += "Version: " + gMapItService.version + "\n";
	debugString += "Map List [" + gMapItService.settings.mapList + "]:\n  ";
	debugString += gMapItService.settings.mappers["$id$"] + "\n";
	debugString += "Mapper \"" + gMapItService.mapper.name + "\" [";
	debugString += gMapItService.mapper.url + "]:\n  ";
	debugString += gMapItService.mapper.id + "\n";
	debugString += "Core: " + gMapItCoreId + "\n";
	debugString += "Service: " + gMapItService.id + "\n";
	debugString += "Settings: " + gMapItService.settings.id + "\n";
	debugString += "Utils: $Id: mapitUtil.js,v 1.8 2005/07/20 22:46:51 jheddings Exp $\n";
	debugString += "**** END MapIt! DEBUG LOG ****";
	return debugString;
}

////////////////////////////////////////////////////////////////////////////////
// Copies the specified text to the clipboard.
// from MakeLink extension

function setClipboardText(text) {
	gMapItService.log("set clipboard:\n" + text);

	var clip = Components.classes['@mozilla.org/widget/clipboard;1']
						 .createInstance(Components.interfaces.nsIClipboard);
	if (! clip) { return false; }

	var trans = Components.classes['@mozilla.org/widget/transferable;1']
							.createInstance(Components.interfaces.nsITransferable);
	if (! trans) { return false; }

	trans.addDataFlavor("text/unicode");
	var str = Components.classes["@mozilla.org/supports-string;1"]
						.createInstance(Components.interfaces.nsISupportsString);

	str.data = text;
	trans.setTransferData("text/unicode", str, text.length * 2);

	var clipid = Components.interfaces.nsIClipboard;
	clip.setData(trans, null, clipid.kGlobalClipboard);
}

////////////////////////////////////////////////////////////////////////////////
// Count number of open browser windows

function countBrowserWindows() {
  var wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
           .getService(Components.interfaces.nsIWindowMediator);

  var browsers = wm.getZOrderDOMWindowEnumerator('navigator:browser', true);

  var count = 0;
  while (browsers.hasMoreElements()) {
    var browser = browsers.getNext();
    count++;
  }

  return count;
}